! This program models acoustic wave propagation in tilted orthorhombic media using rotated non-staggered finite difference of 10th order according to Chu (SEG 2012 Extended Abstract) "A hybrid finite difference method for acoustic wave propagation in tilted orthorhombic media". Equations and parameters are referred from the mentioned paper.
 
program smodeling_main

  use smodeling_mod
  use sep
  use ieee_arithmetic
  use ieee_exceptions

  implicit none
  
  real :: x, y, z, t, dx, dy, dz, dt, eps1, eps2, del1, del2, del3, vpz
  real :: varphi, the, phi
  real, pointer :: cur(:,:,:,:), prev(:,:,:,:), next(:,:,:,:), field(:,:), sou(:)
  integer :: it, nx, ny, nz, nt, ns, ix, iy

  call sep_init() ! SEP initialization function

  ! Initialize physical dimensions x, y, z, t, grid dimensions nx, ny, nz, nt, and time and spatial steps
  call from_param("x", x)
  call from_param("y", y)
  call from_param("z", z)
  call from_param("t", t)
  call from_param("dx", dx)
  call from_param("dy", dy)
  call from_param("dz", dz)
  call from_param("dt", dt)

  nx = x/dx
  ny = y/dy
  nz = z/dz
  nt = t/dt
  
  ! Write out header of output wave field
  call to_history("n1",nx)
  call to_history("n2",ny)
  call to_history("d3",dt)
  call to_history("d1",dx)
  call to_history("d2",dy)
  call to_history("o1",0)
  call to_history("o2",0)
  call to_history("o3",0)

  call to_history("n1",nx,"cube")
  call to_history("n2",ny,"cube")
  call to_history("n3",nz,"cube")
  call to_history("d3",dt,"cube")
  call to_history("d1",dx,"cube")
  call to_history("d2",dy,"cube")
  call to_history("o1",0,"cube")
  call to_history("o2",0,"cube")
  call to_history("o3",0,"cube")

  ! Allocate and initialize previous, current, and next wavefield
  allocate(cur(nx,ny,nz,3), prev(nx,ny,nz,3), next(nx,ny,nz,3))
  allocate(field(nx,ny)) ! to store the wave movie
  cur = 0.
  prev = 0.
  next = 0.
  field = 0.
  
  ! Anisotropic elastic parameters
  call from_param("vpz", vpz)
  call from_param("eps1", eps1)
  call from_param("eps2", eps2)
  call from_param("del1", del1)
  call from_param("del2", del2)
  call from_param("del3", del3)
  call from_param("varphi", varphi)
  call from_param("the", the)
  call from_param("phi", phi)
    
  ! Initialize grid dimensions, time and spatial steps for the module
  call init(dx, dy, dz, dt, nx, ny, nz, nt)
  
  ! Initialize "velocity"
  call init_vel(eps1, eps2, del1, del2, del3, vpz, varphi, the, phi)

  ! Read in Ricker source from command line
  call from_aux("source", "n1", ns)
  allocate(sou(ns))
  call sep_read(sou, "source")

  ! Initialize coefficients of 8th order finite difference
  call init_c8()
  call init_s8()

  ! Loop through every time step
  do it=1,nt
    write(0,*) it

    if (it <= ns) then
      cur(nx/2,ny/2,nz/2,:) = cur(nx/2,ny/2,nz/2,:) + sou(it) ! Source injection
    else 
      cur(nx/2,ny/2,nz/2,:) = cur(nx/2,ny/2,nz/2,:) 
    endif

    call one_step8(prev, cur, next)

    if (maxval(abs(next)) > 1.e17) then
      write(0,*) "blow up"
      write(0,*) maxval(abs(next))
      exit  ! Exit the loop when the amplitude blows up
    endif

    if (.not. ieee_is_finite(sum(next))) then
      write(0,*) "nan"
      exit ! Exit the loop when NaN is found
    endif

    ! store one instance of wave field at z = nz/2
    field(:,:) = -(next(:,:,nz/2,1)+next(:,:,nz/2,2)+next(:,:,nz/2,3))/3. 
    call sep_write(field) ! Write out the wavefield

    prev = cur ! Update wavefield
    cur = next
  enddo

  call to_history("n3",it)

  call sep_write(-(next(:,:,:,1)+next(:,:,:,2)+next(:,:,:,3))/3.,"cube")
 
  deallocate(cur, prev, next, field, sou)

  call sep_close() ! SEP closing function

end program
